syntax = "proto2";

package protos_generator.tests;

import "protos_generator/tests/no_package.proto";

// option java_multiple_files = true;

message MyMessage {
  optional MessageWithEnumUpbTest my_type = 1;
}