syntax = "proto3";

package xds.core.v3;

option java_outer_classname = "ExtensionProto";
option java_multiple_files = true;
option java_package = "com.github.xds.core.v3";
option go_package = "github.com/cncf/xds/go/xds/core/v3";

import "validate/validate.proto";
import "google/protobuf/any.proto";

// Message type for extension configuration.
message TypedExtensionConfig {
  // The name of an extension. This is not used to select the extension, instead
  // it serves the role of an opaque identifier.
  string name = 1 [(validate.rules).string = {min_len: 1}];

  // The typed config for the extension. The type URL will be used to identify
  // the extension. In the case that the type URL is *xds.type.v3.TypedStruct*
  // (or, for historical reasons, *udpa.type.v1.TypedStruct*), the inner type
  // URL of *TypedStruct* will be utilized. See the
  // :ref:`extension configuration overview
  // <config_overview_extension_configuration>` for further details.
  google.protobuf.Any typed_config = 2 [(validate.rules).any = {required: true}];
}
