/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/trace/v3/trace.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/config/trace/v3/trace.upb.h"
#include "envoy/config/trace/v3/datadog.upb.h"
#include "envoy/config/trace/v3/dynamic_ot.upb.h"
#include "envoy/config/trace/v3/http_tracer.upb.h"
#include "envoy/config/trace/v3/lightstep.upb.h"
#include "envoy/config/trace/v3/opencensus.upb.h"
#include "envoy/config/trace/v3/opentelemetry.upb.h"
#include "envoy/config/trace/v3/service.upb.h"
#include "envoy/config/trace/v3/zipkin.upb.h"

// Must be last.
#include "upb/port/def.inc"

const upb_MiniTableFile envoy_config_trace_v3_trace_proto_upb_file_layout = {
  NULL,
  NULL,
  NULL,
  0,
  0,
  0,
};

#include "upb/port/undef.inc"

