syntax = "proto3";

package xds.annotations.v3;

import "google/protobuf/descriptor.proto";

option go_package = "github.com/cncf/xds/go/xds/annotations/v3";

extend google.protobuf.MessageOptions {
  // Magic number is the 28 most significant bits in the sha256sum of
  // "xds.annotations.v3.versioning".
  VersioningAnnotation versioning = 92389011;
}

message VersioningAnnotation {
  // Track the previous message type. E.g. this message might be
  // xds.foo.v3alpha.Foo and it was previously xds.bar.v2.Bar. This
  // information is consumed by UDPA via proto descriptors.
  string previous_message_type = 1;
}
