# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc.beta._connectivity_channel."""

import unittest

from grpc.beta import interfaces


class ConnectivityStatesTest(unittest.TestCase):
    def testBetaConnectivityStates(self):
        self.assertIsNotNone(interfaces.ChannelConnectivity.IDLE)
        self.assertIsNotNone(interfaces.ChannelConnectivity.CONNECTING)
        self.assertIsNotNone(interfaces.ChannelConnectivity.READY)
        self.assertIsNotNone(interfaces.ChannelConnectivity.TRANSIENT_FAILURE)
        self.assertIsNotNone(interfaces.ChannelConnectivity.FATAL_FAILURE)


if __name__ == "__main__":
    unittest.main(verbosity=2)
