// Copyright 2022 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Local copy of Envoy xDS proto file, used for testing only.

syntax = "proto3";

package envoy.extensions.load_balancing_policies.wrr_locality.v3;

import "src/proto/grpc/testing/xds/v3/cluster.proto";

// [#protodoc-title: Weighted Round Robin Locality-Picking Load Balancing Policy]

// Configuration for the wrr_locality LB policy. See the :ref:`load balancing architecture overview
// <arch_overview_load_balancing_types>` for more information.
// [#extension: envoy.clusters.lb_policy]
message WrrLocality {
  // The child LB policy to create for endpoint-picking within the chosen locality.
  config.cluster.v3.LoadBalancingPolicy endpoint_picking_policy = 1;
}
