//
// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

// Local copy of Envoy xDS proto file, used for testing only.

syntax = "proto3";

package envoy.type.matcher.v3;

import "src/proto/grpc/testing/xds/v3/string.proto";

// [#protodoc-title: Path matcher]

// Specifies the way to match a path on HTTP request.
message PathMatcher {
  oneof rule {
    // The `path` must match the URL path portion of the :path header. The query and fragment
    // string (if present) are removed in the URL path portion.
    // For example, the path */data* will match the *:path* header */data#fragment?param=value*.
    StringMatcher path = 1;
  }
}
