// Copyright 2016 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto2";

import "src/proto/grpc/testing/proto2/empty2.proto";

package grpc.testing.proto2;

// Fill emptiness with music.
extend grpc.testing.proto2.EmptyWithExtensions {
  optional int64 Deadmau5 = 124;
  optional float Madeon = 125;
  optional string AboveAndBeyond = 126;
  optional bool Tycho = 127;
  optional fixed64 Pendulum = 128;
}
