// Copyright 2023 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef BSSL_FILLINS_FILE_UTIL_H
#define BSSL_FILLINS_FILE_UTIL_H

#include <openssl/base.h>

#include "path_service.h"

#include <string>

namespace bssl {

namespace fillins {

OPENSSL_EXPORT bool ReadFileToString(const FilePath &path, std::string *out);

}  // namespace fillins

}  // namespace bssl

#endif  // BSSL_FILLINS_FILE_UTIL_H
